//
// Copyright (c) 2015 Assured Information Security, Inc
//
// Dates Modified:
//  - 4/8/2015: Initial commit
//    Rian Quinn <quinnr@ainfosec.com>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "qemu/osdep.h"
#include "qemu-common.h"
#include "audio.h"
#include "qemu/timer.h"
#include "alsadef.h"

///
/// Convert QEMU audio settings to ALSA audio settings
///
/// @param fmt the QEMU audio format
/// @param endianness the QEMU audio endianness
///
/// @return the ALSA equivalent audio format
///
snd_pcm_format_t aud_to_alsafmt(audfmt_e fmt, int endianness)
{
    switch (fmt) {
        case AUD_FMT_S8:
            return SND_PCM_FORMAT_S8;

        case AUD_FMT_U8:
            return SND_PCM_FORMAT_U8;

        case AUD_FMT_S16:
            if (endianness) {
                return SND_PCM_FORMAT_S16_BE;
            }
            else {
                return SND_PCM_FORMAT_S16_LE;
            }

        case AUD_FMT_U16:
            if (endianness) {
                return SND_PCM_FORMAT_U16_BE;
            }
            else {
                return SND_PCM_FORMAT_U16_LE;
            }

        case AUD_FMT_S32:
            if (endianness) {
                return SND_PCM_FORMAT_S32_BE;
            }
            else {
                return SND_PCM_FORMAT_S32_LE;
            }

        case AUD_FMT_U32:
            if (endianness) {
                return SND_PCM_FORMAT_U32_BE;
            }
            else {
                return SND_PCM_FORMAT_U32_LE;
            }

        default:
            return SND_PCM_FORMAT_U8;
    }
}

///
/// Convert ALSA audio format to QEMU audio format
///
/// @param alsafmt the ALSA audio format
/// @param fmt place to store the QEMU audio format equivalent
/// @param endianness place to store the QEMU audio endianness equivalent
///
/// @return -1 on error
///         0 on success
///
int alsa_to_audfmt(snd_pcm_format_t alsafmt, audfmt_e *fmt, int *endianness)
{
    switch (alsafmt) {
        case SND_PCM_FORMAT_S8:
            *endianness = 0;
            *fmt = AUD_FMT_S8;
            break;

        case SND_PCM_FORMAT_U8:
            *endianness = 0;
            *fmt = AUD_FMT_U8;
            break;

        case SND_PCM_FORMAT_S16_LE:
            *endianness = 0;
            *fmt = AUD_FMT_S16;
            break;

        case SND_PCM_FORMAT_U16_LE:
            *endianness = 0;
            *fmt = AUD_FMT_U16;
            break;

        case SND_PCM_FORMAT_S16_BE:
            *endianness = 1;
            *fmt = AUD_FMT_S16;
            break;

        case SND_PCM_FORMAT_U16_BE:
            *endianness = 1;
            *fmt = AUD_FMT_U16;
            break;

        case SND_PCM_FORMAT_S32_LE:
            *endianness = 0;
            *fmt = AUD_FMT_S32;
            break;

        case SND_PCM_FORMAT_U32_LE:
            *endianness = 0;
            *fmt = AUD_FMT_U32;
            break;

        case SND_PCM_FORMAT_S32_BE:
            *endianness = 1;
            *fmt = AUD_FMT_S32;
            break;

        case SND_PCM_FORMAT_U32_BE:
            *endianness = 1;
            *fmt = AUD_FMT_U32;
            break;

        default:
            return -1;
    }

    return 0;
}
